"""Discover Homekit devices."""
from . import MDNSDiscoverable

from ..const import ATTR_NAME


class Discoverable(MDNSDiscoverable):
    """Add support for discovering HomeKit devices."""

    def __init__(self, nd):
        super(Discoverable, self).__init__(nd, '_hap._tcp.local.')

    def info_from_entry(self, entry):
        info = super(Discoverable, self).info_from_entry(entry)
        name = entry.name
        name = name.replace('._hap._tcp.local.', '')
        info[ATTR_NAME] = name
        return info



"""The tests for discovering Xbox gaming consoles via SmartGlass protocol."""
import unittest
from binascii import unhexlify

from netdisco.smartglass import XboxSmartGlass


class TestXboxOne(unittest.TestCase):
    """Test the Xbox One Discoverable."""
    def setUp(self):
        """
        Setup test class
        """
        with open('tests/xboxone_files/discovery_response', 'rb') as content:
            packet = content.read()

        if not packet:
            raise Exception('Failed to read test data')

        self.discovery_response = packet

    def test_assemble_request(self):
        """
        Test discovery request assembly
        """
        packet = XboxSmartGlass.discovery_packet()

        self.assertEqual(
            packet,
            unhexlify(b'dd00000a000000000000000400000002')
        )

    def test_parse_response(self):
        """
        Test discovery response parsing
        """
        response = XboxSmartGlass.parse_discovery_response(
            self.discovery_response)

        self.assertEqual(response['device_type'], 1)
        self.assertEqual(response['flags'], 2)
        self.assertEqual(response['name'], 'XboxOne')
        self.assertEqual(response['uuid'],
                         'DE305D54-75B4-431B-ADB2-EB6B9E546014')
        self.assertEqual(response['last_error'], 0)
        self.assertEqual(response['certificate'][:8], '30820203')
        self.assertEqual(len(unhexlify(response['certificate'])), 519)

    def test_verify_response(self):
        """
        Test discovery response verification
        """
        valid_parse = XboxSmartGlass.verify_packet(self.discovery_response)
        invalid_length = XboxSmartGlass.verify_packet(unhexlify(b'41'))
        invalid_magic = XboxSmartGlass.verify_packet(
            unhexlify(b'aabbccddeeff00'))

        self.assertIsNotNone(valid_parse)
        self.assertIsNone(invalid_length)
        self.assertIsNone(invalid_magic)



"""Discover Lutron Caseta Smart Bridge and Smart Bridge Pro devices."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Lutron Caseta Smart Bridge
    and Smart Bridge Pro devices."""

    def __init__(self, nd):
        """Initialize the Lutron Smart Bridge discovery."""
        super(Discoverable, self).__init__(nd, '_lutron._tcp.local.')



"""Discover Samsung Printers"""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Support for the discovery of Samsung Printers"""

    def get_entries(self):
        """Get all the Samsung Printer uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "Samsung Electronics",
            "deviceType": "urn:schemas-upnp-org:device:Printer:1"
        })



"""Discover Songpal devices."""
import logging
from . import SSDPDiscoverable
from . import ATTR_PROPERTIES


class Discoverable(SSDPDiscoverable):
    """Support for Songpal devices.
    Supported devices: http://vssupport.sony.net/en_ww/device.html."""

    def get_entries(self):
        """Get all the Songpal devices."""
        devs = self.find_by_st(
            "urn:schemas-sony-com:service:ScalarWebAPI:1")

        # At least some Bravia televisions use the same API for communication,
        # but are handled by another platforms, so we filter them out here.
        supported = []
        for dev in devs:
            if 'device' in dev.description:
                device = dev.description['device']
                scalarweb_info = device.get("X_ScalarWebAPI_DeviceInfo", None)

                if scalarweb_info:
                    services = scalarweb_info["X_ScalarWebAPI_ServiceList"]
                    service_types = services["X_ScalarWebAPI_ServiceType"]
                    # Sony Bravias offer videoScreen service, soundbars do not
                    if 'videoScreen' in service_types:
                        continue

                supported.append(dev)

        return supported

    def info_from_entry(self, entry):
        """Get information for a device.."""
        info = super().info_from_entry(entry)

        cached_descs = entry.DESCRIPTION_CACHE[entry.location]

        device_info_element = "X_ScalarWebAPI_DeviceInfo"
        baseurl_element = "X_ScalarWebAPI_BaseURL"
        device_element = "device"
        if device_element in cached_descs and \
                device_info_element in cached_descs[device_element]:
            scalarweb = cached_descs[device_element][device_info_element]

            properties = {"scalarwebapi": scalarweb}
            if baseurl_element in scalarweb:
                properties["endpoint"] = scalarweb[baseurl_element]
            else:
                logging.warning("Unable to find %s", baseurl_element)
            info[ATTR_PROPERTIES] = properties
        else:
            logging.warning("Unable to find ScalarWeb element from desc.")

        return info



"""Provide service that scans the network in intervals."""
import logging
import threading
import time
from collections import defaultdict
from typing import Any, Callable, Dict, List  # noqa: F401

from .discovery import NetworkDiscovery

DEFAULT_INTERVAL = 300  # seconds

_LOGGER = logging.getLogger(__name__)


class DiscoveryService(threading.Thread):
    """Service that will scan the network for devices each `interval` seconds.

    Add listeners to the service to be notified of new services found.
    """

    def __init__(self, interval=DEFAULT_INTERVAL):
        """Initialize the discovery."""
        super(DiscoveryService, self).__init__()

        # Scanning interval
        self.interval = interval

        # Listeners for new services
        self.listeners = []  # type: List[Callable[[str, Any], None]]

        # To track when we have to stop
        self._stop = threading.Event()

        # Tell Python not to wait till this thread exits
        self.daemon = True

        # The discovery object
        self.discovery = None

        # Dict to keep track of found services. We do not want to
        # broadcast the same found service twice.
        self._found = defaultdict(list)  # type: Dict[str, List]

    def add_listener(self, listener):
        """Add a listener for new services."""
        self.listeners.append(listener)

    def stop(self):
        """Stop the service."""
        self._stop.set()

    def run(self):
        """Start the discovery service."""
        self.discovery = NetworkDiscovery()

        while True:
            self._scan()

            seconds_since_scan = 0

            while seconds_since_scan < self.interval:
                if self._stop.is_set():
                    return

                time.sleep(1)
                seconds_since_scan += 1

    def _scan(self):
        """Scan for new devices."""
        _LOGGER.info("Scanning")
        self.discovery.scan()

        for disc in self.discovery.discover():
            for service in self.discovery.get_info(disc):
                self._service_found(disc, service)

        self.discovery.stop()

    def _service_found(self, disc, service):
        """Tell listeners a service was found."""
        if service not in self._found[disc]:
            self._found[disc].append(service)

            for listener in self.listeners:
                try:
                    listener(disc, service)
                except Exception:  # pylint: disable=broad-except
                    _LOGGER.exception(
                        "Error calling listener")



"""Discover Philips Hue bridges."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Philips Hue bridges."""

    def get_entries(self):
        """Get all the Hue bridge uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "Royal Philips Electronics",
            "manufacturerURL": "http://www.philips.com",
            "modelNumber": ["929000226503", "BSB002"]
        })



"""Discover Konnected Security devices."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Konnected Security devices."""

    def get_entries(self):
        """Return all Konnected entries."""
        return self.find_by_st('urn:schemas-konnected-io:device:Security:1')



"""The tests for discovering Yamaha Receivers."""
import unittest
from unittest.mock import MagicMock
import xml.etree.ElementTree as ElementTree

from netdisco.discoverables.yamaha import Discoverable
from netdisco.util import etree_to_dict

LOCATION = 'http://192.168.XXX.XXX:80/desc.xml'


class MockUPNPEntry(object):
    """UPNPEntry backed by a description file."""

    location = LOCATION

    def __init__(self, name):
        """Read and parse a MockUPNPEntry from a file."""
        with open('tests/discoverables/yamaha_files/%s' % name,
                  encoding='utf-8') as content:
            self.description = etree_to_dict(
                ElementTree.fromstring(content.read())).get('root', {})


class TestYamaha(unittest.TestCase):
    """Test the Yamaha Discoverable."""

    def test_info_from_entry_rx_v481(self):
        self.assertEqual(
            Discoverable(None).info_from_entry(
                MockUPNPEntry("desc_RX-V481.xml")),
            {
                'control_url':
                'http://192.168.XXX.XXX:80/YamahaRemoteControl/ctrl',
                'description_url':
                'http://192.168.XXX.XXX:80/YamahaRemoteControl/desc.xml',
                'host': '192.168.xxx.xxx',
                'model_name': 'RX-V481',
                'model_number': 'V481',
                'manufacturer': 'Yamaha Corporation',
                'name': 'RX-V481 XXXXXX',
                'port': 80,
                'serial': 'XXXXXXXX',
                'ssdp_description': 'http://192.168.XXX.XXX:80/desc.xml',
                'udn': 'uuid:XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX',
                'upnp_device_type':
                'urn:schemas-upnp-org:device:MediaRenderer:1',
            })

    def test_info_from_entry_single_service(self):
        self.assertEqual(
            Discoverable(None).info_from_entry(
                MockUPNPEntry("desc_single_service.xml")),
            {
                'control_url':
                'http://192.168.1.2:80/YamahaRemoteControl/single_ctrl',
                'description_url':
                'http://192.168.1.2:80/YamahaRemoteControl/single_desc.xml',
                'host': '192.168.xxx.xxx',
                'model_name': 'RX-V single service model name',
                'model_number': None,
                'manufacturer': None,
                'name': 'single service friendly name',
                'port': 80,
                'serial': None,
                'ssdp_description': 'http://192.168.XXX.XXX:80/desc.xml',
                'udn': None,
                'upnp_device_type': None,
            })

    def test_info_from_entry_multiple_services_remote_control_last(self):
        self.assertEqual(
            Discoverable(None).info_from_entry(MockUPNPEntry(
                "desc_multiple_services_remote_control_last.xml")),
            {
                'control_url':
                'http://192.168.1.2:80/YamahaRemoteControl/multi_ctrl',
                'description_url':
                'http://192.168.1.2:80/YamahaRemoteControl/multi_desc.xml',
                'host': '192.168.xxx.xxx',
                'model_name': 'RX-V multi service model name',
                'model_number': None,
                'manufacturer': None,
                'name': 'multi service friendly name',
                'port': 80,
                'serial': None,
                'ssdp_description': 'http://192.168.XXX.XXX:80/desc.xml',
                'udn': None,
                'upnp_device_type': None,
            })

    def test_info_from_entry_multiple_services_no_remote_control(self):
        self.assertEqual(
            Discoverable(None).info_from_entry(MockUPNPEntry(
                "desc_multiple_services_no_remote_control.xml")),
            {
                'control_url':
                'http://192.168.1.2:80/YamahaNewControl/ctrl',
                'description_url':
                'http://192.168.1.2:80/YamahaNewControl/desc.xml',
                'host': '192.168.xxx.xxx',
                'model_name': 'RX-V multi service model name',
                'model_number': None,
                'manufacturer': None,
                'name': 'multi service friendly name',
                'port': 80,
                'serial': None,
                'ssdp_description': 'http://192.168.XXX.XXX:80/desc.xml',
                'udn': None,
                'upnp_device_type': None,
            })

    def test_get_entries_incompatible_models(self):
        supported_model = MockUPNPEntry(
            "desc_RX-V481.xml")
        devices = [
            supported_model,
            MockUPNPEntry("desc_R-N602.xml")
        ]

        discoverable = Discoverable(None)
        discoverable.find_by_device_description = MagicMock(
            return_value=devices)

        self.assertEqual(discoverable.get_entries(), [supported_model])



""" Discover Cambridge Audio StreamMagic devices. """
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Cambridge Audio StreamMagic devices."""

    def get_entries(self):
        """Get all Cambridge Audio MediaRenderer uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "Cambridge Audio",
            "deviceType": "urn:schemas-upnp-org:device:MediaRenderer:1"
        })



"""Daikin device discovery."""
import socket

from datetime import timedelta
from typing import Dict, List  # noqa: F401
from urllib.parse import unquote

DISCOVERY_MSG = b"DAIKIN_UDP/common/basic_info"

UDP_SRC_PORT = 30000
UDP_DST_PORT = 30050

DISCOVERY_ADDRESS = '<broadcast>'
DISCOVERY_TIMEOUT = timedelta(seconds=2)


class Daikin:
    """Base class to discover Daikin devices."""

    def __init__(self):
        """Initialize the Daikin discovery."""
        self.entries = []  # type: List[Dict[str, str]]

    def scan(self):
        """Scan the network."""
        self.update()

    def all(self):
        """Scan and return all found entries."""
        self.scan()
        return self.entries

    def update(self):
        """Scan network for Daikin devices."""
        entries = []

        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.settimeout(DISCOVERY_TIMEOUT.seconds)
        sock.bind(("", UDP_SRC_PORT))

        try:

            sock.sendto(DISCOVERY_MSG, (DISCOVERY_ADDRESS, UDP_DST_PORT))

            while True:
                try:
                    data, (address, _) = sock.recvfrom(1024)

                    entry = {x[0]: x[1] for x in (
                        e.split('=', 1)
                        for e in data.decode("UTF-8").split(','))}

                    # expecting product, mac, activation code, version
                    if 'ret' not in entry or entry['ret'] != 'OK':
                        # non-OK return on response
                        continue

                    if 'mac' not in entry:
                        # no mac found for device"
                        continue

                    if 'type' not in entry or entry['type'] != 'aircon':
                        # no mac found for device"
                        continue

                    if 'name' in entry:
                        entry['name'] = unquote(entry['name'])

                    # in case the device was not configured to have an id
                    # then use the mac address
                    if 'id' in entry and entry['id'] == '':
                        entry['id'] = entry['mac']

                    entries.append({
                        'id': entry['id'],
                        'name': entry['name'],
                        'ip': address,
                        'mac': entry['mac'],
                        'ver': entry['ver'],
                    })

                except socket.timeout:
                    break

        finally:
            sock.close()

        self.entries = entries


def main():
    """Test Daikin discovery."""
    from pprint import pprint
    daikin = Daikin()
    pprint("Scanning for Daikin devices..")
    daikin.update()
    pprint(daikin.entries)


if __name__ == "__main__":
    main()



"""Combine all the different protocols into a simple interface."""
import logging
import os
import importlib

from .ssdp import SSDP
from .mdns import MDNS
from .gdm import GDM
from .lms import LMS
from .tellstick import Tellstick
from .daikin import Daikin
from .smartglass import XboxSmartGlass

_LOGGER = logging.getLogger(__name__)


class NetworkDiscovery:
    """Scan the network for devices.

    mDNS scans in a background thread.
    SSDP scans in the foreground.
    GDM scans in the foreground.
    LMS scans in the foreground.
    Tellstick scans in the foreground
    Xbox One scans in the foreground

    start: is ready to scan
    scan: scan the network
    discover: parse scanned data
    get_in
    """

    # pylint: disable=too-many-instance-attributes
    def __init__(self):
        """Initialize the discovery."""

        self.mdns = None
        self.ssdp = None
        self.gdm = None
        self.lms = None
        self.tellstick = None
        self.daikin = None
        self.xbox_smartglass = None

        self.is_discovering = False
        self.discoverables = None

    def scan(self, zeroconf_instance=None, suppress_mdns_types=None):
        """Start and tells scanners to scan."""
        self.is_discovering = True

        self.mdns = MDNS(zeroconf_instance)

        # Needs to be after MDNS init
        self._load_device_support()

        if suppress_mdns_types:
            for type_ in suppress_mdns_types:
                self.mdns.unregister_type(type_)

        self.mdns.start()

        self.ssdp = SSDP()
        self.ssdp.scan()

        self.gdm = GDM()
        self.gdm.scan()

        self.lms = LMS()
        self.lms.scan()

        self.tellstick = Tellstick()
        self.tellstick.scan()

        self.daikin = Daikin()
        self.daikin.scan()

        self.xbox_smartglass = XboxSmartGlass()
        self.xbox_smartglass.scan()

    def stop(self):
        """Turn discovery off."""
        if not self.is_discovering:
            return

        self.mdns.stop()

        # Not removing SSDP because it tracks state
        self.mdns = None
        self.gdm = None
        self.lms = None
        self.tellstick = None
        self.daikin = None
        self.xbox_smartglass = None
        self.discoverables = None
        self.is_discovering = False

    def discover(self):
        """Return a list of discovered devices and services."""
        if not self.is_discovering:
            raise RuntimeError("Needs to be called after start, before stop")

        return [dis for dis, checker in self.discoverables.items()
                if checker.is_discovered()]

    def get_info(self, dis):
        """Get a list with the most important info about discovered type."""
        return self.discoverables[dis].get_info()

    def get_entries(self, dis):
        """Get a list with all info about a discovered type."""
        return self.discoverables[dis].get_entries()

    def _load_device_support(self):
        """Load the devices and services that can be discovered."""
        self.discoverables = {}

        discoverables_format = __name__.rsplit('.', 1)[0] + '.discoverables.{}'

        for module_name in os.listdir(os.path.join(os.path.dirname(__file__),
                                                   'discoverables')):
            if module_name[-3:] != '.py' or module_name == '__init__.py':
                continue

            module_name = module_name[:-3]

            module = importlib.import_module(
                discoverables_format.format(module_name))

            self.discoverables[module_name] = \
                getattr(module, 'Discoverable')(self)

    def print_raw_data(self):
        """Helper method to show what is discovered in your network."""
        from pprint import pprint

        print("Zeroconf")
        pprint(self.mdns.entries)
        print("")
        print("SSDP")
        pprint(self.ssdp.entries)
        print("")
        print("GDM")
        pprint(self.gdm.entries)
        print("")
        print("LMS")
        pprint(self.lms.entries)
        print("")
        print("Tellstick")
        pprint(self.tellstick.entries)
        print("")
        print("Xbox SmartGlass")
        pprint(self.xbox_smartglass.entries)



"""Discover IGD services."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering IGD services."""

    def get_entries(self):
        """Get all the IGD service uPnP entries."""
        return \
            self.find_by_st(
                "urn:schemas-upnp-org:device:InternetGatewayDevice:1") + \
            self.find_by_st(
                "urn:schemas-upnp-org:device:InternetGatewayDevice:2")



"""Discover Ziggo Mediabox XL devices."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Ziggo Mediabox XL devices."""

    def get_entries(self):
        """Return all Ziggo (UPC) Mediabox XL entries."""
        return self.find_by_device_description(
            {'modelDescription': 'UPC Hzn Gateway',
             'deviceType': 'urn:schemas-upnp-org:device:RemoteUIServer:2'})



"""
Example use of DiscoveryService.

Will scan every 10 seconds and print out new found entries.
Will quit after 2 minutes.

"""
import logging
from datetime import datetime
import time

from netdisco.service import DiscoveryService

logging.basicConfig(level=logging.INFO)

# Scan every 10 seconds
nd = DiscoveryService(10)


def new_service_listener(discoverable, service):
    """ Print out a new service found message. """
    print("{} - Found new service: {} {}".format(
        datetime.now(), discoverable, service))


nd.add_listener(new_service_listener)

nd.start()

time.sleep(120)

nd.stop()



"""Discover Roku players."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Roku media players."""

    def get_entries(self):
        """Get all the Roku entries."""
        return self.find_by_st("roku:ecp")



"""Tellstick device discovery."""
import socket
from datetime import timedelta
import logging
from typing import List, Tuple  # noqa: F401


DISCOVERY_PORT = 30303
DISCOVERY_ADDRESS = '<broadcast>'
DISCOVERY_PAYLOAD = b"D"
DISCOVERY_TIMEOUT = timedelta(seconds=2)


class Tellstick:
    """Base class to discover Tellstick devices."""

    def __init__(self):
        """Initialize the Tellstick discovery."""
        self.entries = []  # type: List[Tuple[str]]

    def scan(self):
        """Scan the network."""
        self.update()

    def all(self):
        """Scan and return all found entries."""
        self.scan()
        return self.entries

    def update(self):
        """Scan network for Tellstick devices."""
        entries = []

        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.settimeout(DISCOVERY_TIMEOUT.seconds)
        sock.sendto(DISCOVERY_PAYLOAD, (DISCOVERY_ADDRESS, DISCOVERY_PORT))

        while True:
            try:
                data, (address, _) = sock.recvfrom(1024)
                entry = data.decode("ascii").split(":")
                # expecting product, mac, activation code, version
                if len(entry) != 4:
                    continue
                entry.insert(0, address)
                entries.append(tuple(entry))

            except socket.timeout:
                break
            except UnicodeDecodeError:
                # Catch invalid responses
                logging.getLogger(__name__).debug(
                    'Ignoring invalid unicode response from %s', address)
                continue

            self.entries = entries

        sock.close()


def main():
    """Test Tellstick discovery."""
    from pprint import pprint
    tellstick = Tellstick()
    pprint("Scanning for Tellstick devices..")
    tellstick.update()
    pprint(tellstick.entries)


if __name__ == "__main__":
    main()



"""Discover TiVo DVR devices providing the TCP Remote Protocol."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering TiVo Remote Protocol service."""

    def __init__(self, nd):
        """Initialize the discovery.

        Yields a dictionary with hostname, host and port along with a
        properties sub-dictionary with some device specific ids.
        """
        super(Discoverable, self).__init__(nd, '_tivo-remote._tcp.local.')



"""Discover DLNA services."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering DLNA services."""

    def get_entries(self):
        """Get all the DLNA service uPnP entries."""
        return self.find_by_st("urn:schemas-upnp-org:device:MediaServer:1") + \
            self.find_by_st("urn:schemas-upnp-org:device:MediaServer:2") + \
            self.find_by_st("urn:schemas-upnp-org:device:MediaServer:3") + \
            self.find_by_st("urn:schemas-upnp-org:device:MediaServer:4")



"""Discover Canon Printers"""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Support for the discovery of Canon Printers"""

    def get_entries(self):
        """Get all the Canon Printer uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "CANON INC.",
            "deviceType": "urn:schemas-cipa-jp:device:DPSPrinter:1"
        })



"""Squeezebox/Logitech Media server discovery."""
import socket
from typing import Dict, List, Union  # noqa: F401

from .const import ATTR_HOST, ATTR_PORT

DISCOVERY_PORT = 3483
DEFAULT_DISCOVERY_TIMEOUT = 2


class LMS:
    """Base class to discover Logitech Media servers."""

    def __init__(self):
        """Initialize the Logitech discovery."""
        self.entries = []  # type: List[Dict[str, Union[str, int]]]
        self.last_scan = None

    def scan(self):
        """Scan the network."""
        self.update()

    def all(self):
        """Scan and return all found entries."""
        self.scan()
        return list(self.entries)

    def update(self):
        """Scan network for Logitech Media Servers."""
        lms_ip = '<broadcast>'
        lms_port = DISCOVERY_PORT
        lms_msg = b"eJSON\0"
        lms_timeout = DEFAULT_DISCOVERY_TIMEOUT

        entries = []

        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.settimeout(lms_timeout)
        sock.bind(('', 0))

        try:
            sock.sendto(lms_msg, (lms_ip, lms_port))

            while True:
                try:
                    data, server = sock.recvfrom(1024)
                    if data.startswith(b'E'):
                        # Full response is EJSON\xYYXXXX
                        # Where YY is length of port string (ie 4)
                        # And XXXX is the web interface port (ie 9000)
                        port = None
                        # https://github.com/python/typeshed/pull/2696
                        if data.startswith(b'JSON', 1):  # type: ignore
                            length = data[5:6][0]
                            port = int(data[0-length:])
                        entries.append({
                            ATTR_HOST: server[0],
                            ATTR_PORT: port,
                        })
                except socket.timeout:
                    break
        finally:
            sock.close()
        self.entries = entries


def main():
    """Test LMS discovery."""
    from pprint import pprint

    lms = LMS()

    pprint("Scanning for Logitech Media Servers...")
    lms.update()
    pprint(lms.entries)


if __name__ == "__main__":
    main()



"""Discover Samsung Smart TV services."""
from . import SSDPDiscoverable
from ..const import ATTR_NAME

# For some models, Samsung forces a [TV] prefix to the user-specified name.
FORCED_NAME_PREFIX = '[TV]'


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Samsung Smart TV services."""

    def get_entries(self):
        """Get all the Samsung RemoteControlReceiver entries."""
        return self.find_by_st(
            "urn:samsung.com:device:RemoteControlReceiver:1")

    def info_from_entry(self, entry):
        """Get most important info, by default the description location."""
        info = super().info_from_entry(entry)

        # Strip the forced prefix, if present
        if info[ATTR_NAME].startswith(FORCED_NAME_PREFIX):
            info[ATTR_NAME] = info[ATTR_NAME][len(FORCED_NAME_PREFIX):].strip()

        return info



"""Add support for discovering mDNS services."""
import itertools
import logging
from typing import List  # noqa: F401

from zeroconf import DNSPointer, DNSRecord
from zeroconf import Error as ZeroconfError
from zeroconf import ServiceBrowser, ServiceInfo, ServiceStateChange, Zeroconf

_LOGGER = logging.getLogger(__name__)


class FastServiceBrowser(ServiceBrowser):
    """ServiceBrowser that does not process record updates."""

    def update_record(self, zc: Zeroconf, now: float, record: DNSRecord) -> None:
        """Ignore record updates for non-ptrs."""
        if record.name not in self.types or not isinstance(record, DNSPointer):
            return
        super().update_record(zc, now, record)


class MDNS:
    """Base class to discover mDNS services."""

    def __init__(self, zeroconf_instance=None):
        """Initialize the discovery."""
        self.zeroconf = zeroconf_instance
        self._created_zeroconf = False
        self.services = []  # type: List[ServiceInfo]
        self._browser = None  # type: ServiceBrowser

    def register_service(self, service):
        """Register a mDNS service."""
        self.services.append(service)

    def unregister_type(self, type_):
        """Unregister a mDNS type."""
        removes = []
        for service in self.services:
            if service.typ == type_:
                removes.append(service)
        for service in removes:
            self.services.remove(service)

    def start(self):
        """Start discovery."""
        try:
            if not self.zeroconf:
                self.zeroconf = Zeroconf()
                self._created_zeroconf = True

            services_by_type = {}

            for service in self.services:
                services_by_type.setdefault(service.typ, [])
                services_by_type[service.typ].append(service)

            def _service_update(zeroconf, service_type, name, state_change):
                if state_change == ServiceStateChange.Added:
                    for service in services_by_type[service_type]:
                        try:
                            service.add_service(zeroconf, service_type, name)
                        except ZeroconfError:
                            _LOGGER.exception("Failed to add service %s", name)
                elif state_change == ServiceStateChange.Removed:
                    for service in services_by_type[service_type]:
                        service.remove_service(zeroconf, service_type, name)

            types = [service.typ for service in self.services]
            self._browser = FastServiceBrowser(
                self.zeroconf, types, handlers=[_service_update]
            )
        except Exception:  # pylint: disable=broad-except
            self.stop()
            raise

    def stop(self):
        """Stop discovering."""
        if self._browser:
            self._browser.cancel()
            self._browser = None

        for service in self.services:
            service.reset()

        if self._created_zeroconf:
            self.zeroconf.close()
            self.zeroconf = None

    @property
    def entries(self):
        """Return all entries in the cache."""
        return list(
            itertools.chain(
                *[
                    self.zeroconf.cache.entries_with_name(name)
                    for name in self.zeroconf.cache.names()
                ]
            )
        )



"""Discover OctoPrint Servers."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering OctoPrint servers."""

    def get_entries(self):
        """Get all the OctoPrint uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "The OctoPrint Project"
        })



"""Discover Wink hub devices."""
from typing import List  # noqa: F401

from . import SSDPDiscoverable
from ..ssdp import UPNPEntry  # noqa: F401


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Wink hub devices."""

    def get_entries(self):
        """Return all Wink entries."""
        results = []  # type: List[UPNPEntry]
        results.extend(self.find_by_st('urn:wink-com:device:hub2:2'))
        results.extend(self.find_by_st('urn:wink-com:device:hub:2'))
        results.extend(self.find_by_st('urn:wink-com:device:relay:2'))
        return results



"""Discover Apple TV media players."""
from . import MDNSDiscoverable
from ..const import ATTR_NAME, ATTR_PROPERTIES


class Discoverable(MDNSDiscoverable):
    """Add support for Apple TV devices."""

    def __init__(self, nd):
        super(Discoverable, self).__init__(nd, '_appletv-v2._tcp.local.')

    def info_from_entry(self, entry):
        """Returns most important info from mDNS entries."""
        info = super().info_from_entry(entry)
        info[ATTR_NAME] = info[ATTR_PROPERTIES]['Name'].replace('\xa0', ' ')
        return info



"""
Discover Sercomm network cameras.
These are rebranded as iControl and many others, and are usually
distributed as part of an ADT or Comcast/Xfinity monitoring package.
https://github.com/edent/Sercomm-API
"""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering camera services."""

    def get_entries(self):
        """Get all Sercomm iControl devices."""
        return self.find_by_device_description({'manufacturer': 'iControl'})



"""
Support for discovery using GDM (Good Day Mate), multicast protocol by Plex.

Inspired by
  hippojay's plexGDM:
  https://github.com/hippojay/script.plexbmc.helper/resources/lib/plexgdm.py
  iBaa's PlexConnect: https://github.com/iBaa/PlexConnect/PlexAPI.py
"""
import socket
import struct
from typing import Any, Dict, List  # noqa: F401


class GDM:
    """Base class to discover GDM services."""

    def __init__(self):
        self.entries = []  # type: List[Dict[str, Any]]
        self.last_scan = None

    def scan(self):
        """Scan the network."""
        self.update()

    def all(self):
        """Return all found entries.

        Will scan for entries if not scanned recently.
        """
        self.scan()
        return list(self.entries)

    def find_by_content_type(self, value):
        """Return a list of entries that match the content_type."""
        self.scan()
        return [entry for entry in self.entries
                if value in entry['data']['Content_Type']]

    def find_by_data(self, values):
        """Return a list of entries that match the search parameters."""
        self.scan()
        return [entry for entry in self.entries
                if all(item in entry['data'].items()
                       for item in values.items())]

    def update(self):
        """Scan for new GDM services.

        Example of the dict list assigned to self.entries by this function:
        [{'data': {
             'Content-Type': 'plex/media-server',
             'Host': '53f4b5b6023d41182fe88a99b0e714ba.plex.direct',
             'Name': 'myfirstplexserver',
             'Port': '32400',
             'Resource-Identifier': '646ab0aa8a01c543e94ba975f6fd6efadc36b7',
             'Updated-At': '1444852697',
             'Version': '0.9.12.13.1464-4ccd2ca',
          },
          'from': ('10.10.10.100', 32414)}]
        """

        gdm_ip = '239.0.0.250'  # multicast to PMS
        gdm_port = 32414
        gdm_msg = 'M-SEARCH * HTTP/1.0'.encode('ascii')
        gdm_timeout = 1

        self.entries = []

        # setup socket for discovery -> multicast message
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.settimeout(gdm_timeout)

        # Set the time-to-live for messages for local network
        sock.setsockopt(socket.IPPROTO_IP,
                        socket.IP_MULTICAST_TTL,
                        struct.pack("B", gdm_timeout))

        try:
            # Send data to the multicast group
            sock.sendto(gdm_msg, (gdm_ip, gdm_port))

            # Look for responses from all recipients
            while True:
                try:
                    bdata, server = sock.recvfrom(1024)
                    data = bdata.decode('utf-8')
                    if '200 OK' in data.splitlines()[0]:
                        ddata = {k: v.strip() for (k, v) in (
                            line.split(':') for line in
                            data.splitlines() if ':' in line)}
                        self.entries.append({'data': ddata,
                                             'from': server})
                except socket.timeout:
                    break
        finally:
            sock.close()


def main():
    """Test GDM discovery."""
    from pprint import pprint

    gdm = GDM()

    pprint("Scanning GDM...")
    gdm.update()
    pprint(gdm.entries)


if __name__ == "__main__":
    main()



"""Discover Hikvision cameras."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Hikvision cameras."""

    def __init__(self, nd):
        """Initialize Hikvision camera discovery."""
        super(Discoverable, self).__init__(nd, '_http._tcp.local.')

    def get_entries(self):
        return self.find_by_device_name('HIKVISION')



"""Discover Arduino devices."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Arduino devices."""

    def __init__(self, nd):
        super(Discoverable, self).__init__(nd, '_arduino._tcp.local.')



"""Discover Enigma2 servers."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Enigma2 boxes."""

    def __init__(self, nd):
        """Initialize the Enigma2 discovery."""
        super(Discoverable, self).__init__(nd, '_e2stream._tcp.local.')



"""Discover Heos devices."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering DLNA services."""

    def get_entries(self):
        """Get all the HEOS devices."""
        return self.find_by_st("urn:schemas-denon-com:device:ACT-Denon:1")



"""Discover Nanoleaf Aurora devices."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Nanoleaf Aurora devices."""

    def __init__(self, nd):
        super(Discoverable, self).__init__(nd, '_nanoleafapi._tcp.local.')



"""Discover Home Assistant servers."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering mobile apps that support Home Assistant."""

    def __init__(self, nd):
        super(Discoverable, self).__init__(nd, '_hass-mobile-app._tcp.local.')



"""Discover HP Printers"""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Support for the discovery of HP Printers"""

    def __init__(self, nd):
        """Initialize the HP Printer discovery"""
        super(Discoverable, self).__init__(nd, '_printer._tcp.local.')

    def get_entries(self):
        return self.find_by_device_name('HP ')



"""Discover Volumio servers."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Volumio."""

    def __init__(self, nd):
        """Initialize the Volumio discovery."""
        super(Discoverable, self).__init__(nd, '_Volumio._tcp.local.')



"""Discover deCONZ gateways."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering deCONZ Wireless Light Control gateways."""

    def get_entries(self):
        """Get all the deCONZ uPnP entries."""
        return self.find_by_device_description({
            "manufacturerURL": "http://www.dresden-elektronik.de",
            "modelDescription": "dresden elektronik Wireless Light Control"
        })



"""Discover devices that implement the Ikea Tradfri platform."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Ikea Tradfri devices."""

    def __init__(self, nd):
        """Initialize the Cast discovery."""
        super(Discoverable, self).__init__(nd, '_coap._udp.local.')



"""Util functions used by Netdisco."""
from collections import defaultdict
from typing import Any, Dict, List, Optional  # noqa: F401


# Taken from http://stackoverflow.com/a/10077069
# pylint: disable=invalid-name
def etree_to_dict(t):
    """Convert an ETree object to a dict."""
    # strip namespace
    tag_name = t.tag[t.tag.find("}")+1:]

    d = {
        tag_name: {} if t.attrib else None
    }  # type: Dict[str, Optional[Dict[str, Any]]]
    children = list(t)
    if children:
        dd = defaultdict(list)  # type: Dict[str, List]
        for dc in map(etree_to_dict, children):
            for k, v in dc.items():
                dd[k].append(v)
        d = {tag_name: {k: v[0] if len(v) == 1 else v for k, v in dd.items()}}
    dt = d[tag_name]
    if t.attrib:
        assert dt is not None
        dt.update(('@' + k, v) for k, v in t.attrib.items())
    if t.text:
        text = t.text.strip()
        if children or t.attrib:
            if text:
                assert dt is not None
                dt['#text'] = text
        else:
            d[tag_name] = text
    return d



"""Discover Huawei routers."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Huawei routers."""

    def get_entries(self):
        """Get all the Huawei uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "Huawei Technologies Co., Ltd.",
            "deviceType": "urn:schemas-upnp-org:device:InternetGatewayDevice:1"
        })



"""Discover ESPHome devices."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering ESPHome devices."""

    def __init__(self, nd):
        super().__init__(nd, '_esphomelib._tcp.local.')



"""Discover Xiaomi Mi Home (aka Lumi) Gateways."""
from . import MDNSDiscoverable
from ..const import ATTR_MAC_ADDRESS, ATTR_PROPERTIES


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Xiaomi Gateway"""

    def __init__(self, nd):
        """Initialize the discovery."""
        super(Discoverable, self).__init__(nd, '_miio._udp.local.')

    def info_from_entry(self, entry):
        """Return most important info from mDNS entries."""
        info = super().info_from_entry(entry)

        # Workaround of misparsing of mDNS properties. It's unclear
        # whether it's bug in zeroconf module or in the Gateway, but
        # returned properties look like:
        # {b'poch': b'0:mac=286c07aaaaaa\x00'} instead of expected:
        # {b'epoch': b'0', b'mac': '286c07aaaaaa'}
        if "poch" in info[ATTR_PROPERTIES]:
            misparsed = info[ATTR_PROPERTIES]["poch"]
            misparsed = misparsed.rstrip("\0")
            for val in misparsed.split(":"):
                if val.startswith("mac="):
                    info[ATTR_MAC_ADDRESS] = val[len("mac="):]

        return info

    def get_entries(self):
        """Return Xiaomi Gateway devices."""
        return self.find_by_device_name('lumi-gateway-')



"""Discover Freebox routers."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Freebox routers."""

    def __init__(self, nd):
        """Initialize the Freebox discovery."""
        super(Discoverable, self).__init__(nd, '_fbx-api._tcp.local.')



"""Discover Harmony Hub remotes."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering Harmony Hub remotes"""

    def get_entries(self):
        """Get all the Harmony uPnP entries."""
        return self.find_by_device_description({
            "manufacturer": "Logitech",
            "deviceType": "urn:myharmony-com:device:harmony:1"
        })



"""Discover Xbox SmartGlass devices."""
from . import BaseDiscoverable


class Discoverable(BaseDiscoverable):
    """Add support for discovering a Xbox SmartGlass device."""

    def __init__(self, netdis):
        """Initialize the Xbox SmartGlass discovery."""
        self._netdis = netdis

    def get_entries(self):
        """Get all the Xbox SmartGlass details."""
        return self._netdis.xbox_smartglass.entries



"""Discover Axis devices."""
from . import MDNSDiscoverable

from ..const import (
    ATTR_HOST, ATTR_PORT, ATTR_HOSTNAME, ATTR_PROPERTIES)


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Axis devices."""

    def info_from_entry(self, entry):
        """Return most important info from mDNS entries."""
        properties = {}

        for key, value in entry.properties.items():
            if isinstance(value, bytes):
                value = value.decode('utf-8')
            properties[key.decode('utf-8')] = value

        return {
            ATTR_HOST: self.ip_from_host(entry.server),
            ATTR_PORT: entry.port,
            ATTR_HOSTNAME: entry.server,
            ATTR_PROPERTIES: properties,
        }

    def __init__(self, nd):
        """Initialize the Axis discovery."""
        super(Discoverable, self).__init__(nd, '_axis-video._tcp.local.')

    def ip_from_host(self, host):
        """Attempt to return the ip address from an mDNS host.

        Return host if failed.
        """
        ips = self.netdis.mdns.zeroconf.cache.entries_with_name(host.lower())

        try:
            return repr(ips[0]) if ips else host
        except TypeError:
            return host



"""Setup file for netdisco."""
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, "README.md"), encoding="utf-8") as readme_file:
    long_description = readme_file.read()

setup(
    name="netdisco",
    version="2.9.0",
    description="Discover devices on your local network",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/home-assistant/netdisco",
    author="Paulus Schoutsen",
    author_email="Paulus@PaulusSchoutsen.nl",
    license="Apache License 2.0",
    install_requires=["requests>=2.0", "zeroconf>=0.30.0"],
    python_requires=">=3",
    packages=find_packages(exclude=["tests", "tests.*"]),
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        "Topic :: Home Automation",
        "Topic :: System :: Networking",
    ],
)



"""Discover Yeelight bulbs, based on Kodi discoverable."""
from . import MDNSDiscoverable
from ..const import ATTR_DEVICE_TYPE

DEVICE_NAME_PREFIX = 'yeelink-light-'


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Yeelight."""

    def __init__(self, nd):
        """Initialize the Yeelight discovery."""
        super(Discoverable, self).__init__(nd, '_miio._udp.local.')

    def info_from_entry(self, entry):
        """Return most important info from mDNS entries."""
        info = super().info_from_entry(entry)

        # Example name: yeelink-light-ceiling4_mibt72799069._miio._udp.local.
        info[ATTR_DEVICE_TYPE] = \
            entry.name.replace(DEVICE_NAME_PREFIX, '').split('_', 1)[0]

        return info

    def get_entries(self):
        """ Return yeelight devices. """
        return self.find_by_device_name(DEVICE_NAME_PREFIX)



"""Discover devices that implement the Bluesound platform."""
from . import MDNSDiscoverable


class Discoverable(MDNSDiscoverable):
    """Add support for discovering Bluesound service."""

    def __init__(self, nd):
        """Initialize the Bluesound discovery."""
        super(Discoverable, self).__init__(nd, '_musc._tcp.local.')



"""Discover AVM FRITZ devices."""
from . import SSDPDiscoverable


class Discoverable(SSDPDiscoverable):
    """Add support for discovering AVM FRITZ devices."""

    def get_entries(self):
        """Get all AVM FRITZ entries."""
        return self.find_by_st("urn:schemas-upnp-org:device:fritzbox:1")



"""Xbox One SmartGlass device discovery."""
import socket
import struct
import binascii
from datetime import timedelta
from typing import Any, Dict, List, Optional, Tuple  # noqa: F401


DISCOVERY_PORT = 5050
DISCOVERY_ADDRESS_BCAST = '<broadcast>'
DISCOVERY_ADDRESS_MCAST = '239.255.255.250'
DISCOVERY_REQUEST = 0xDD00
DISCOVERY_RESPONSE = 0xDD01
DISCOVERY_TIMEOUT = timedelta(seconds=2)

"""
SmartGlass Client type
XboxOne = 1
Xbox360 = 2
WindowsDesktop = 3
WindowsStore = 4
WindowsPhone = 5
iPhone = 6
iPad = 7
Android = 8
"""
DISCOVERY_CLIENT_TYPE = 4

_Response = Dict[str, Any]


class XboxSmartGlass:
    """Base class to discover Xbox SmartGlass devices."""

    def __init__(self):
        """Initialize the Xbox SmartGlass discovery."""
        self.entries = []  # type: List[Tuple[str, Optional[_Response]]]
        self._discovery_payload = self.discovery_packet()

    @staticmethod
    def discovery_packet():
        """Assemble discovery payload."""
        version = 0
        flags = 0
        min_version = 0
        max_version = 2

        payload = struct.pack(
            '>IHHH',
            flags, DISCOVERY_CLIENT_TYPE, min_version, max_version
        )
        header = struct.pack(
            '>HHH',
            DISCOVERY_REQUEST, len(payload), version
        )
        return header + payload

    @staticmethod
    def parse_discovery_response(data):
        """Parse console's discovery response."""
        pos = 0
        # Header
        # pkt_type, payload_len, version = struct.unpack_from(
        #    '>HHH',
        #    data, pos
        # )
        pos += 6
        # Payload
        flags, type_, name_len = struct.unpack_from(
            '>IHH',
            data, pos
        )
        pos += 8
        name = data[pos:pos + name_len]
        pos += name_len + 1  # including null terminator
        uuid_len = struct.unpack_from(
            '>H',
            data, pos
        )[0]
        pos += 2
        uuid = data[pos:pos + uuid_len]
        pos += uuid_len + 1  # including null terminator
        last_error, cert_len = struct.unpack_from(
            '>IH',
            data, pos
        )
        pos += 6
        cert = data[pos:pos + cert_len]

        return {
            'device_type': type_,
            'flags': flags,
            'name': name.decode('utf-8'),
            'uuid': uuid.decode('utf-8'),
            'last_error': last_error,
            'certificate': binascii.hexlify(cert).decode('utf-8')
        }

    def scan(self):
        """Scan the network."""
        self.update()

    def all(self):
        """Scan and return all found entries."""
        self.scan()
        return self.entries

    @staticmethod
    def verify_packet(data):
        """Parse packet if it has correct magic"""
        if len(data) < 2:
            return None

        pkt_type = struct.unpack_from('>H', data)[0]
        if pkt_type != DISCOVERY_RESPONSE:
            return None

        return XboxSmartGlass.parse_discovery_response(data)

    def update(self):
        """Scan network for Xbox SmartGlass devices."""
        entries = []

        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.settimeout(DISCOVERY_TIMEOUT.seconds)
        sock.sendto(self._discovery_payload,
                    (DISCOVERY_ADDRESS_BCAST, DISCOVERY_PORT))
        sock.sendto(self._discovery_payload,
                    (DISCOVERY_ADDRESS_MCAST, DISCOVERY_PORT))

        while True:
            try:
                data, (address, _) = sock.recvfrom(1024)

                response = self.verify_packet(data)
                if response:
                    entries.append((address, response))

            except socket.timeout:
                break

            self.entries = entries

        sock.close()


def main():
    """Test XboxOne discovery."""
    from pprint import pprint
    xbsmartglass = XboxSmartGlass()
    pprint("Scanning for Xbox One SmartGlass consoles devices..")
    xbsmartglass.update()
    pprint(xbsmartglass.entries)


if __name__ == "__main__":
    main()
